@@ -1,10 +1,13 @@
+from io import StringIO
 import allure
+import boto3
 import pandas as pd
 import time, os
 import logging
 # from matplotlib import pyplot as plt
 # import numpy as np
 # import seaborn as sn
+import s3fs
 from toolkit_w.internal.api_requestor import APIRequestor
 from toolkit_w.internal.token_json import UserToken
 from toolkit_w.internal.vars_global import ProblemType, TargetMetric, FeatureType
@@ -27,39 +30,45 @@ class MyWhatify:
     #############################
     user_token = None
 
-    def __init__(self):
+    @classmethod
+    def __init__(cls):
         pass
 
-    def authenticate(self, username, password):
-        self.login(email=username,password=password)
+    @classmethod
+    def authenticate(cls, username, password):
+        cls.login(email=username, password=password)
 
-    def login(self, email, password, user_id: str = None):
+    @classmethod
+    def login(cls, email, password, user_id: str = None):
         with allure.step('login with user: ' + str(email)):
             logging.info('login with user: ' + str(email))  # + ' and password: ' + password)
             my_toolkit.UserManagement.login(email=email, password=password)
             if user_id:
                 with allure.step(' '.join(['impersonate to user_ID:', str(user_id)])):
                     my_toolkit.UserManagement.impersonate(user_id=user_id)
-            self.user_token = UserToken(my_toolkit.token)
+            cls.user_token = UserToken(my_toolkit.token)
 
-
-    def get_user_id(self, email: str = None):
+    @classmethod
+    def get_user_id(cls, email: str = None):
         user_id = -1
         with allure.step('get user ID for current user or for user: ' + str(email)):
             logging.info('get user ID for current user or for user: ' + str(email))
             if email:
+                # my_toolkit.UserManagement.li
                 raise Exception('Need to implement')
                 user_id = 1
             else:
-                user_id = self.user_token.get_user_id()
+                user_id = cls.user_token.get_user_id()
 
         return user_id
 
-    def impersonate(self, user_id, email: str = None, admin_token: str = None):
+    @classmethod
+    def impersonate(cls, user_id, email: str = None, admin_token: str = None):
         with allure.step(' '.join(['impersonate to user_ID:', str(user_id)])):
             my_toolkit.UserManagement.impersonate(email=email, user_id=user_id, admin_token=admin_token)
 
-    def mark_ensemble_in_production(self, ensemble_id, mark_production=True):
+    @classmethod
+    def mark_ensemble_in_production(cls, ensemble_id, mark_production=True):
         with allure.step('setting ensemble_id: ' + str(ensemble_id) + ' as in production: ' + str(mark_production)):
             logging.info('setting ensemble_id: ' + str(ensemble_id) + ' as in production: ' + str(mark_production))
             if mark_production:
@@ -67,8 +76,8 @@ class MyWhatify:
             else:
                 my_toolkit.Ensemble.remove_ensemble_in_production(ensemble_id=ensemble_id)
 
-
-    def run_prediction(self, data_name, ensemble_id, data_file=None, wait=True, file_path=None, header=None, data_id=None):
+    @classmethod
+    def run_prediction(cls, data_name, ensemble_id, data_file=None, wait=True, file_path=None, header=None, data_id=None):
         with allure.step('running prediction on data name: ' + data_name + ' with ensemble id: ' + str(ensemble_id) + ' and wait is: ' + str(wait)):
             print('running prediction on data name: ' + data_name + ' with ensemble id: ' + str(ensemble_id) + ' and wait is: ' + str(wait))
             logging.info('running prediction on data name: ' + data_name + ' with ensemble id: ' + str(ensemble_id) + ' and wait is: ' + str(wait))
@@ -79,33 +88,42 @@ class MyWhatify:
             try:
                 if not (file_path or data_id):
                     data_id = my_toolkit.Datasource.create_from_dataframe(df=data_file, data_source_name=data_name,
-                                                                       wait=True, skip_if_exists=True)['id']
+                                                                          wait=True, skip_if_exists=True)['id']
                 predict_id = my_toolkit.Prediction.create(ensemble_id=ensemble_id, data_id=data_id, data_name=data_name,
-                                                       file_path=file_path, header=header, wait=wait)['id']
+                                                          file_path=file_path, header=header, wait=wait)['id']
             except Exception as ex:
                 message = ex.args
                 status = False
         return predict_id, status, message
 
-    def upload_data_file(self, data_file, data_name, wait=True, skip_if_exists=True, na_values=None):
+    @classmethod
+    def run_ice(cls, pred_id):
+        return my_toolkit.Prediction.run_ice(pred_id=pred_id)
+
+    @classmethod
+    def run_prescriptive(cls, pred_id, features, target_value='YES'):
+        return my_toolkit.Prediction.run_prescriptive(pred_id=pred_id, features=features, target_value=target_value)
+
+    @classmethod
+    def upload_data_file(cls, data_file, data_name, wait=True, skip_if_exists=True, na_values=None):
         with allure.step('uploading csv name: ' + data_name + ' with wait= ' + str(wait) +
                      ' and skip if exist= ' + str(skip_if_exists)):
             source_id = -1
             try:
-                source_id = my_toolkit.Datasource.create_from_dataframe(df=data_file,
-                                                                     data_source_name=data_name,
-                                                                     wait=wait, skip_if_exists=skip_if_exists,
-                                                                     na_values=na_values)['id']
+                source_id = my_toolkit.Datasource.create_from_dataframe(df=data_file, data_source_name=data_name,
+                                                                        wait=wait, skip_if_exists=skip_if_exists,
+                                                                        na_values=na_values)['id']
                 print("\nYour Source ID for {} is: {}".format(data_name, source_id))
             except Exception as e:
                 print(e)
         return source_id
 
-    def preparing_data_set(self, source_id, data_set_name, sample_id=['car_id'],
+    @classmethod
+    def preparing_data_set(cls, source_id, data_set_name, sample_id=['car_id'],
                            retype_columns={'car_id': FeatureType.CATEGORICAL}, target='class',
                            problem_type=ProblemType.CLASSIFICATION, wait=True):
         with allure.step('Preparing Dataset name: ' + str(data_set_name) + ' source_id= ' + str(source_id) +
-                     ' problem_type: ' + str(problem_type) + ' target: ' + str(target) + ' wait: ' + str(wait)):
+                         ' problem_type: ' + str(problem_type) + ' target: ' + str(target) + ' wait: ' + str(wait)):
             dataset_id = -1
             # task_type = eval(problem_type.value)
 
@@ -120,32 +138,35 @@ class MyWhatify:
                     wait=wait, skip_if_exists=True)['id']
                 print("Dataset {} id is {}".format(data_set_name, dataset_id))
             except Exception as e:
-                logging.exception('failed preparing data set' + data_set_name +' got error: ' + e)
+                logging.exception('failed preparing data set' + data_set_name + ' got error: ' + e)
         return dataset_id
 
-    def create_data_set(self, source_id, data_set_name, problem_type=ProblemType.CLASSIFICATION, target='class',
+    @classmethod
+    def create_data_set(cls, source_id, data_set_name, problem_type=ProblemType.CLASSIFICATION, target='class',
                         not_used=None, na_values=None, skip_if_exists=True, wait=True):
         with allure.step('Preparing Dataset name: ' + str(data_set_name) + ' source_id= ' + str(source_id) +
-                     ' problem_type: ' + str(problem_type) + ' target: ' + str(target) + ' wait: ' + str(wait)):
+                         ' problem_type: ' + str(problem_type) + ' target: ' + str(target) + ' wait: ' + str(wait)):
             dataset_id = -1
             task_type = eval(problem_type.value)
 
             try:
                 dataset_id = my_toolkit.Dataset.create(datasource_id=source_id, dataset_name=data_set_name,
-                                    problem_type=task_type,
-                                    target=target, not_used=not_used, na_values=na_values,
-                                    skip_if_exists=skip_if_exists, wait=True)['id']
+                                                       problem_type=task_type, target=target, not_used=not_used,
+                                                       na_values=na_values,  skip_if_exists=skip_if_exists,
+                                                       wait=True)['id']
                 print("Dataset {} id is {}".format(data_set_name, dataset_id))
             except Exception as e:
-                logging.exception('failed creating data set' + data_set_name +' got error: ' + e)
+                logging.exception('failed creating data set' + data_set_name + ' got error: ' + e)
         return dataset_id
 
-    def train_model(self, data_name, dataset_id, target_metric=TargetMetric.RECALL_MACRO, training_time_in_minutes=6,
+    @classmethod
+    def train_model(cls, data_name, dataset_id, target_metric=TargetMetric.RECALL_MACRO, training_time_in_minutes=6,
                     cv_folds=3, ensemble_size=5, interpt_level=Interpt.EXPLAINABLE):
-        with allure.step('On data_name: ' + data_name + ' ID: ' + str(dataset_id) +
-                     ' training a model with target_metric: ' + str(target_metric.value) +
-                     ' cv_folds: ' + str(cv_folds) + ' ensemble_size: ' + str(ensemble_size) +
-                     ' interpt_level: ' + str(interpt_level) + ' training_time_in_minutes:' + str(training_time_in_minutes)):
+        with allure.step('On data_name: ' + data_name + ' ID: ' + str(dataset_id)
+                         + ' training a model with target_metric: ' + str(target_metric.value)
+                         + ' cv_folds: ' + str(cv_folds) + ' ensemble_size: ' + str(ensemble_size)
+                         + ' interpt_level: ' + str(interpt_level) + ' training_time_in_minutes:'
+                         + str(training_time_in_minutes)):
             task_id = None
             ret_dict = None
             # target_metric_eval = eval(target_metric.value)
@@ -155,8 +176,8 @@ class MyWhatify:
             # @markdown * Data partitioning
             # @markdown * Hold-out // Cross-Validation
 
-            pipeline = self.get_pipeline_estimators_list(dataset_id, True, interpt_level)
-            estimators = self.get_pipeline_estimators_list(dataset_id, False, interpt_level)
+            pipeline = cls.get_pipeline_estimators_list(dataset_id, True, interpt_level)
+            estimators = cls.get_pipeline_estimators_list(dataset_id, False, interpt_level)
             if interpt_level == Interpt.EXPLAINABLE:
                 interpretability_level = my_toolkit.enums.InterpretabilityLevel.EXPLAINABLE
             else:
@@ -183,12 +204,15 @@ class MyWhatify:
                 task_id = task['id']
                 ret_dict = task.to_dict()
             except Exception as e:
-                logging.exception('Failed to train model on data_name: ' + str(data_name) + ' dataset_id: ' + str(dataset_id) +
-                                  ' target_metric_eval ' + str(target_metric_eval) + 'in Time: ' +
-                                  str(training_time_in_minutes) + ' interpretability_level ' + str(interpretability_level))
+                logging.exception('Failed to train model on data_name: ' + str(data_name)
+                                  + ' dataset_id: ' + str(dataset_id)
+                                  # + ' target_metric_eval ' + str(target_metric_eval)
+                                  + 'in Time: ' + str(training_time_in_minutes)
+                                  + ' interpretability_level ' + str(interpretability_level))
         return task_id, ret_dict
 
-    def model_sensitivity_report(self, ensemble_id, alg='Permutation', max_iteration=10):
+    @classmethod
+    def model_sensitivity_report(cls, ensemble_id, alg='Permutation', max_iteration=10):
         logging.info('running model sensitivity report for ensemble_id: ' + str(ensemble_id) + ' alg: ' + str(alg) +
                      ' max iter: ' + str(max_iteration))
         df = pd.DataFrame()
@@ -210,88 +234,102 @@ class MyWhatify:
 
         return sens[alg]
 
-    def delete_data_source(self, data_id):
+    @classmethod
+    def delete_data_source(cls, data_id):
         logging.info('deleting data Source data_id: ' + str(data_id))
         my_toolkit.Datasource.delete(data_id)
 
-    def delete_data_set(self, data_id):
+    @classmethod
+    def delete_data_set(cls, data_id):
         logging.info('deleting dataset data_id: ' + str(data_id))
         my_toolkit.Dataset.delete(data_id)
 
-    def delete_task(self, task_id):
+    @classmethod
+    def delete_task(cls, task_id):
         logging.info('deleting task_id: ' + str(task_id))
         my_toolkit.Task.delete(task_id)
 
-    # def delete_user(self, email=None, password=None, delete_user_id=None):
+    # def delete_user(cls, email=None, password=None, delete_user_id=None):
     #     with allure.step('Marking user email ' + str(email) + ' or user_id:' + str(delete_user_id) + ' as Deleted in the DB with API'):
     #         logging.info('deleting user email: ' + str(email))
     #         if not delete_user_id:
-    #             self.login(email=email, password=password)
-    #             delete_user_id = self.user_token.get_user_id()
+    #             cls.login(email=email, password=password)
+    #             delete_user_id = cls.user_token.get_user_id()
     #
     #         if delete_user_id:
-    #             self.login(email=vars_global.Users.AUTO_QA_ADMIN.get_email(), password=vars_global.Users.AUTO_QA_ADMIN.get_password())
+    #             cls.login(email=vars_global.Users.AUTO_QA_ADMIN.get_email(), password=vars_global.Users.AUTO_QA_ADMIN.get_password())
     #             #TODO move to test
-    #             # self.umClient = UMClient(host=conf_server['USERM_HOST'], port=conf_server['USERM_PORT'])  # , use_https=True)
-    #             # self.umClient.delete_user(jwt=self.user_token.get_token(), user_id=delete_user_id)
+    #             # cls.umClient = UMClient(host=conf_server['USERM_HOST'], port=conf_server['USERM_PORT'])  # , use_https=True)
+    #             # cls.umClient.delete_user(jwt=cls.user_token.get_token(), user_id=delete_user_id)
 
     #############################
     ###        Gettrs         ###
     #############################
-    def get_api_post(self, url, api_key=None):
+    @classmethod
+    def get_api_post(cls, url, api_key=None):
         return APIRequestor().post(url=url, api_key=api_key)
 
-    def get_data_source(self, data_name):
+    @classmethod
+    def get_data_source(cls, data_name):
         logging.info('getting the DataSource list for data_name: ' + str(data_name))
         return my_toolkit.Datasource.list(search_term=data_name)['hits']
 
-    def get_data_source_list(self, len=10):
+    @classmethod
+    def get_data_source_list(cls, len=10):
         with allure.step('getting the DataSource list with len: ' + str(len)):
             list_sources = my_toolkit.Datasource.list()
             sources = pd.DataFrame(list_sources['hits'])
 
         return sources[['creation_date', 'data_size', 'id', 'name', 'row_count', 'state', ]].head(len)
 
-    def get_data_set(self, data_name):
+    @classmethod
+    def get_data_set(cls, data_name):
         logging.info('getting the DataSet list for data_name: ' + str(data_name))
         return my_toolkit.Dataset.list(search_term=data_name)['hits']
 
-    def get_datasets_list(self, len=10):
+    @classmethod
+    def get_datasets_list(cls, len=10):
         with allure.step('getting the Dataset list with len: ' + str(len)):
             list_datasets = my_toolkit.Dataset.list()
             datasets = pd.DataFrame(list_datasets['hits'])
             datasets.set_index('creation_date')
         return datasets[['creation_date', 'id', 'name', 'problem_type', 'row_count', 'state']].head(len)
 
-    def get_tasks(self, data_name):
+    @classmethod
+    def get_tasks(cls, data_name):
         logging.info('getting the Tasks list for data_name: ' + str(data_name))
         return my_toolkit.Task.list(search_term=data_name)['hits']
 
-    def get_tasks_list(self, len=10, search_term=None):
+    @classmethod
+    def get_tasks_list(cls, len=10, search_term=None):
         with allure.step('getting the tasks list with len: ' + str(len)):
             list_tasks = my_toolkit.Task.list(search_term=search_term)
             tasks = pd.DataFrame(list_tasks['hits'])
         return tasks[['creation_date', 'dataset_id', 'name', 'notes', 'problem_type', 'state', 'target_metric']].head(len)
 
-    def get_ensemble_list(self, task_id):
+    @classmethod
+    def get_ensemble_list(cls, task_id):
         with allure.step('getting the Ensemble list for task ID: ' + str(task_id)):
             # get list of ensembles
             return my_toolkit.Ensemble.list(filter_={'task_id': [task_id], 'stage': ['TASK', 'REFIT']})['hits']
 
-    def get_ensemble_id(self, task_name):
+    @classmethod
+    def get_ensemble_id(cls, task_name):
         with allure.step('getting the ensemble ID of task name: ' + str(task_name)):
             task_id = my_toolkit.Task.get_task_id(task_name=task_name)
             return my_toolkit.Ensemble.get_ensemble_id(task_id=task_id)
 
-    def get_data_file(self, data_name='car_sdk_demo.csv'):
+    @classmethod
+    def get_data_file(cls, data_name='car_sdk_demo.csv'):
         with allure.step('getting csv from data file for data_name: ' + str(data_name)):
             script_dir = os.path.dirname(__file__)
             path = os.path.join(script_dir, '../data_source/')
             # path = "https://raw.githubusercontent.com/NeuralAlgorithms/firefly-python-sdk/master/examples/"
         return pd.read_csv(path + data_name)
 
-    def get_created_data_file(self, data_name='temp.csv'):
-        df_data = self.get_data_file(data_name='car_sdk_demo_test_no.csv')
+    @classmethod
+    def get_created_data_file(cls, data_name='temp.csv'):
+        df_data = cls.get_data_file(data_name='car_sdk_demo_test_no.csv')
         data = df_data.get_values()
         script_dir = os.path.dirname(__file__)
         path = os.path.join(script_dir, '../data_source/') + data_name
@@ -305,7 +343,8 @@ class MyWhatify:
                     f.write('\n' + text[:-1])
             f.close()
 
-    def get_data_file_s3(self, data_name='car_sdk_demo.csv'):
+    @classmethod
+    def get_data_file_s3(cls, data_name='car_sdk_demo.csv'):
         logging.info('getting csv from data file for data_name: ' + str(data_name))
         script_dir = os.path.dirname(__file__)
         path = os.path.join(script_dir, '../data_source/')
@@ -313,13 +352,43 @@ class MyWhatify:
         # path = "https://raw.githubusercontent.com/NeuralAlgorithms/firefly-python-sdk/master/examples/"
         return pd.read_csv(path + data_name)
 
-    def get_pipeline_list(self, dataset_id, interpt_level=Interpt.PRECISE):
-        return self.get_pipeline_estimators_list(dataset_id, True, interpt_level)
+    @classmethod
+    def ls_client(cls, user_token: str = None):
+        path = my_toolkit.UserManagement.get_client_path(user_token=user_token)
+        fs = s3fs.S3FileSystem()
+        return fs.ls(path)
 
-    def get_pipeline_estimators_list(self, dataset_id, interpt_level=Interpt.PRECISE):
-        return self.get_pipeline_estimators_list(dataset_id, False, interpt_level)
+    @classmethod
+    def get_s3_boto_client(cls, user_token: str = None):
+        credentials = my_toolkit.UserManagement.get_credentials(user_token=user_token)
+        s3_client = boto3.client('s3',
+                                 aws_access_key_id=credentials['access_key'],
+                                 aws_secret_access_key=credentials['secret_key'],
+                                 aws_session_token=credentials['session_token'])
+        return s3_client, credentials['bucket'], credentials['path']
 
-    def get_pipeline_estimators_list(self, dataset_id, is_pipeline=True, interpt_level=Interpt.PRECISE):
+    @classmethod
+    def read_to_pandas_df(cls, s3_client, bucket, path):
+        obj = s3_client.get_object(Bucket=bucket, Key=path)
+        df = pd.read_csv(obj['Body'])
+        return df
+
+    @classmethod
+    def write_pandas_df_to_s3(cls, s3_client, bucket, path, df):
+        csv_buffer = StringIO()
+        df.to_csv(csv_buffer)
+        s3_client.upload_fileobj(csv_buffer, bucket, path)
+
+    @classmethod
+    def get_pipeline_list(cls, dataset_id, interpt_level=Interpt.PRECISE):
+        return cls.get_pipeline_estimators_list(dataset_id, True, interpt_level)
+
+    @classmethod
+    def get_pipeline_estimators_list(cls, dataset_id, interpt_level=Interpt.PRECISE):
+        return cls.get_pipeline_estimators_list(dataset_id, False, interpt_level)
+
+    @classmethod
+    def get_pipeline_estimators_list(cls, dataset_id, is_pipeline=True, interpt_level=Interpt.PRECISE):
         with allure.step('getting the Dataset pipeline or estimators list for dataset_id: ' + str(dataset_id)
                      + ' is_pipeline: ' + str(is_pipeline) + ' interpt_level: ' + str(interpt_level)):
             ret_list = None
@@ -329,7 +398,8 @@ class MyWhatify:
                 ret_list = my_toolkit.Dataset.get_available_estimators(inter_level=interpt_level, id=dataset_id)
         return ret_list
 
-    def get_prediction(self, predict_id, len=10):
+    @classmethod
+    def get_prediction(cls, predict_id, len=10):
         with allure.step('getting prediction list for predict_id: ' + str(predict_id) + ' len: ' + str(len)):
             ret_prediction = None
             predict_results = my_toolkit.Prediction.get(predict_id)
@@ -341,29 +411,31 @@ class MyWhatify:
                 ret_prediction = df_predict.head(len)
         return ret_prediction
 
-    def get_prediction_error(self, predict_id):
+    @classmethod
+    def get_prediction_error(cls, predict_id):
         with allure.step('getting prediction list for predict_id: ' + str(predict_id) + ' len: ' + str(len)):
             error_message = None
             predict_results = my_toolkit.Prediction.get(predict_id)
             error_message = predict_results['error_message']
         return error_message
 
-    def get_wisdom_list(self):
+    @classmethod
+    def get_wisdom_list(cls):
         with allure.step('getting wisdom list'):
             my_list = my_toolkit.Wisdom.get_wisdom_list_with_demo()
             len1 = len(my_list)
         return my_list
 
-    # def get_task_result_Tom(self):
+    # def get_task_result_Tom(cls):
     #     #todo delete fun
-    #     self.login(email=vars_global.Users.AUTO_QA007.get_email(),
+    #     cls.login(email=vars_global.Users.AUTO_QA007.get_email(),
     #                password=vars_global.Users.AUTO_QA007.get_password())
     #
     #     task_id = 138637
     #     task_id_older = 138630
-    #     jwt = self.user_token.get_token()
-    #     self.halClient = HalClient(host=conf_server['HAL_HOST'], port=conf_server['HAL_PORT'], use_https=True)
-    #     self.report_client = ReportsClient(host=conf_server['REPORT_HOST'], port=conf_server['REPORT_PORT'])
+    #     jwt = cls.user_token.get_token()
+    #     cls.halClient = HalClient(host=conf_server['HAL_HOST'], port=conf_server['HAL_PORT'], use_https=True)
+    #     cls.report_client = ReportsClient(host=conf_server['REPORT_HOST'], port=conf_server['REPORT_PORT'])
     #     for index in range(99):
     #         try:
     #             if index % 3:
@@ -374,6 +446,10 @@ class MyWhatify:
     #             pass
     #     print('dddd')
 
+    @classmethod
+    def get_credentials(cls, api_key: str = None):
+        retval = my_toolkit.UserManagement.get_credentials(user_token=api_key)
+        return retval
 
 
     #############################
